/*
 * squash.c
 *		Ver 1.0 (2015/09/25)
 *			ncurses 版
 *		Ver 1.1 (2016/09/23)
 *			ncursesw 版
 *		Ver 1.2 (2017/10/06)
 */

/*
 *	必要なライブラリ : libncursesw5-dev
 *		sudo apt-get install libncursesw5-dev
 *			or make setup
 */

/* ncurses 版
 * #include <curses.h>
 */

#include <ncursesw/ncurses.h>
#include <unistd.h>

/*
 * 画面サイズ
 */

#define	X_WIDTH	60			/* 画面幅 */
#define	Y_HIGHT	20			/* 画面高 */

#define X_MIN	5			/* 画面左端 */
#define	Y_MIN	1			/* 画面下端 */

#define X_MAX	(X_MIN+X_WIDTH-1)		/* 画面右端 */
#define Y_MAX	(Y_MIN+Y_HIGHT-1)		/* 画面上端 */

/*
 * ラケット情報
 */

#define	Y_RACKET	(Y_MAX-2)		/* ラケットの位置 */
#define	RACKET_LEN	8		/* ラケットの長さ */

/*
 * キー情報
 */

#define	GAME_KEY_LEFT	'h'
#define	GAME_KEY_RIGHT	'l'
#define	GAME_KEY_QUIT	'q'



/*
 * 時間情報
 */

#define	W_MSEC	1000		/* 各操作の最小単位時間(小さい程速い) */
#define	W_COUNT	100			/* ボールの移動までの時間 */

/*
 *	report
 *		ボールやラケットの位置を表示する
 */

void report ( int bx, int by, int rx ) {
  move ( 1, X_MAX + 1 );
  refresh();
  printf ( "bx=%d, by=%d rx=%d", bx, by, rx );
}

/*
 * erase_ball
 *		ボールを消す
 */

void erase_ball ( int bx, int by ) {
	move ( by, bx );	/* ボールの位置 (bx,by) にカーソルを移動 */
	addch ( ' ' );		/* ' ' を上書きする事により、ボールを消す */
}

/*
 * draw_ball
 *		ボールを書く
 */

void draw_ball ( int bx, int by ) {
	move ( by, bx );	/* ボールの位置 (bx,by) にカーソルを移動 */
	addch ( '*' );		/* '*' を上書きする事により、ボールを書く */
}

/*
 * erase_racket
 *		ラケットを消す
 */

void erase_racket ( int rx ) {
  int i;

  /* ラケットの長さだけ */
  for ( i = 0; i < RACKET_LEN; i++ ) {
		/* ラケットを作る文字の位置にカーソルを移動 */
	move ( Y_RACKET, rx - RACKET_LEN / 2 + i );
		/* ラケットを消す */
	addch ( ' ' );
  }
}

/*
 * erase_racket
 *		ラケットを書く
 */

void draw_racket ( int rx ) {
  int i;

  /* ラケットの長さだけ */
  for ( i = 0; i < RACKET_LEN; i++ ) {
		/* ラケットを作る文字の位置にカーソルを移動 */
	move ( Y_RACKET, rx - RACKET_LEN / 2 + i );
		/* ラケットを書く */
	addch ( '=' );
  }
}

/*
 * draw_frame
 *		枠を書く
 */

void draw_frame( void ) {
	int x;
	int y;

	for ( y = Y_MIN - 1; y <= Y_MAX + 1; y++ ) {
		move ( y, X_MIN - 1 ); 	addch ( '#' );
		move ( y, X_MAX + 1 ); 	addch ( '#' );
	}

	for ( x = X_MIN - 1; x <= X_MAX + 1; x++ ) {
		move ( Y_MIN - 1, x ); 	addch ( '#' );
		move ( Y_MAX + 1, x ); 	addch ( '#' );
	}

}

/*
 * main
 */

int main(void) {
  int ch;
  int bx = X_MAX/2;		/* ボールの位置の x 座標 (初期値は中央) */
  int by = Y_MAX/2;		/* ボールの位置の y 座標 (初期値は中央) */
  int dbx = 1;			/* ボールの速度の x 成分 (初期値は 1 [右向き] ) */
  int dby = 1;			/* ボールの速度の y 成分 (初期値は 1 [下向き] ) */
  int rx = X_MAX/2;		/* ラケット中央の位置の x 座標 (初期値は中央) */
  int count = 0;		/* ボールが動くまでの待機時間 */

  /* 画面の初期化 */

  initscr();		/* 画面を初期化 (クリアされる) */
  cbreak();			/* 文字の入力をリアルタイムにする */
  noecho();			/* 文字の入力をエコーバックしない */

  timeout(0);		/* 文字の入力がなければ直ぐにかえる */

  /* ゲーム開始 */

  draw_frame();		/* 枠を書く */

  while ( 0 == 0 ) {		/* 無限 Loop */
	usleep ( W_MSEC );		/* W_MSEC だけ待つ (最小時間を一つ進める) */

	count ++;				/* ボールの待機時刻を進める */

	if ( count > W_COUNT ) {	/* ボールの移動時刻が来た */
	  count = 0;				/* 待機時間時刻を最初に戻す */
	  /* report ( bx, by, rx ); */

	  erase_ball( bx, by );		/* 現在あるボールを削除 */

	  /* ボールの x 座標の処理 */

	  bx = bx + dbx;			/* ボールの x 座標を更新 */
	  							/* 「速度」だけ増やす */

	  if ( bx < X_MIN ) {			/* 画面の左端に来た */
		dbx = 1;			/* 「速度」の向きを右向きにする */
		bx = X_MIN + 1;
	  }

	  if ( bx > X_MAX ) {		/* 画面の右端に来た */
		dbx = -1;				/* 「速度」の向きを左向きにする */
		bx = X_MAX - 1;
	  }

	  /* ボールの y 座標の処理 */

	  by = by + dby;			/* ボールの y 座標を更新 */

	  if ( by < Y_MIN ) {		/* 画面の上端に来た */
		dby = 1;				/* 「速度」の向きを下向きにする */
		by = Y_MIN + 1;
	  }

	  if ( by > Y_MAX ) {		/* 画面の下端に来た */
		dby = -1;				/* 「速度」の向きを上向きにする */
		by = Y_MAX - 1;
	  }

	  /* ボールとラケット衝突チェック */

	  /* 衝突した */
	  if ( by == Y_RACKET && rx - RACKET_LEN/2 <= bx && bx <= rx + RACKET_LEN/2 ){
		dby = -1;	/* ボールの速度の向きを「上向き」にする */
	  }

	  draw_ball( bx, by );	/* ボールを表示する */
	}

	/*
	 * 入力処理 (ラケットの移動)
	 */	 

	ch = getch();  			/* 文字の入力 */

	erase_racket ( rx );	/* ラケットを消す */

	/* 入力の内容によって、ラケットの位置を移動 */

	if ( ch == GAME_KEY_LEFT ) {			/* GAME_KEY_LEFT (左方向) が入力された */
	  if ( rx - RACKET_LEN/2 > X_MIN ) {	/* 左端でなければ */
		rx--;					/* 左に移動 */
	  }
	} else if ( ch == GAME_KEY_RIGHT ) {	/* GAME_KEY_RIGHT (右方向) が入力された */
	  if ( rx + RACKET_LEN/2 < X_MAX ) {	/* 右端でなければ */
		rx++;					/* 右に移動 */
	  }
	} else if ( ch == GAME_KEY_QUIT ) {	/* GAME_KEY_QUIT (ゲーム終了) が入力された */
	  break;					/* while loop を抜ける(ゲーム終了) */
	}

	draw_racket ( rx );			/* 新しい位置にラケットを表示する */

	refresh();					/* 書き換えた内容を画面に描画 */
  }

  endwin();						/* curses の後始末 */

  return 0;
}
